fire_demo:
*********************************************************************************
* Small gfx-routine that simulates fire.
* Made by Allan Jensen (2811aj@fiol.brock.dk)
* Version 0.1, I suppose
* 14/11/96
*********************************************************************************
**equates and stuff
start_x:	equ		0					*start of fire (x-coord)
start_y:	equ		600					*start of fire (y-coord)
dim_x:		equ		200					*dimension of fire x
dim_y:		equ		87					*dimension y
s_width:	equ		800					*screen width
black:		equ		0					*definition black
red:		equ		255					*definition red
frequency:	equ		5					*frequency of red (lower = higher freq.)
Random:		equ		$a861				*randomtrap
SetEntries:	equ		$aa3f				*colour routine
buf1_pointer:		globoff.l	1		*pointer to buf1
buf1_handle:		globoff.l	1		*handle to buf1
buf2_pointer:		globoff.l	1		*pointer to buf2
buf2_handle:		globoff.l	1		*handle to buf2
colours:	ds.w	4*255				*space for fire-colours
*********************************************************************************	 	 

		bsr		a5_init					*mandatory
		bsr		init_mac				*mandatory
		bsr		init_graph_lib			*initialise the graphics library (for clear)
		bsr		hide_mouse				*hide the mouse cursor
		bsr		clut_setup				*insert fire-clut

		move.l	#dim_x*dim_y,d0			*get memory for buffer1
		bsr		getmem	
		bsr		lockmem	
		move.l	a0,buf1_handle(a5)		*save for when we need to release it
		move.l	(a0),a0
		move.l	a0,buf1_pointer(a5)

		move.l	#dim_x*dim_y,d0			*get memory for buffer2
		bsr		getmem	
		bsr		lockmem	
		move.l	a0,buf2_handle(a5)
		move.l	(a0),a0
		move.l	a0,buf2_pointer(a5)

		move.b	#black,d0				*colour to clear screen
		bsr		CLS_256					*clear
		bsr		prep_misc				*set up buffers
fire_loop:
		bsr		set_random				*renew fire
		bsr		calc_buffer				*calculates new buffer by average
		bsr		switch_buffer			*switch buffers for plotting
		bsr		draw_screen				*plot buffer
		bsr		getkey_turbo			*get key
		tst.w	d0						*any key will do
		bmi.s	quit					*key pressed
		bra.s	fire_loop
QUIT:	
		move.b	#black,d0
		bsr		CLS_256					*to make screen nice and clean after use
		bsr		show_mouse				*mouse cursor back on
		move.l	buf1_handle(a5),a0		*release reserved memory
		bsr		releasemem
		move.l	buf2_handle(a5),a0
		bsr		releasemem
		rts	

******************* Routines ********************
clut_setup:
		bsr		calc_clut				*calculate my own clut
		bsr		do_CLUT					*put it in
		rts
		
*Calc_clut sets up the nice colours. I've split it into two, so we'll get yellow as well as red
*Play with ratios in d7/d6/d5 to find out what you think is nice 
calc_clut:
		clr.l	d6
		clr.l	d5
		lea.l	colours(pc),a0
		move.w	#24,d7
insert_clut:
		move.w	#$0000,(a0)+			*not sure what to put here, so we just clear it :-)
		move.w	d6,(a0)+				*R
		move.w	#$0000,(a0)+			*G
		move.w	#$0000,(a0)+			*B
		add.w	#250,d6					*add 250 every time to get more red
		dbra	d7,insert_clut
		move.w	#230,d7
insert2:
		move.w	#$0000,(a0)+
		move.w	d6,(a0)+				*R
		move.w	d5,(a0)+				*G
		move.w	#$0000,(a0)+			*B
		add.w	#250,d6
		add.w	#250,d5
		dbra	d7,insert2
		rts		

*Thanks Rob
do_clut:
		move.w	#0,-(sp)				*start at zero
		move.w	#255,-(sp)				*255 entries
		pea		colours(pc)				*push colours
		dc.w	SetEntries				*KABLAM!
		rts


*Pointer to memory-chunks put in variable so they're easier to switch around
prep_misc:
		lea.l	variables(pc),a6		*insert address of buffer
		move.l	buf1_pointer(a5),d0
		move.l	d0,active_screen(a6)
		move.l	buf2_pointer(a5),d0
		move.l	d0,active_buffer(a6)
		rts

switch_buffer:
		lea.l	variables(pc),a6		*remember to switch buffer after each calc
		move.l	active_screen(a6),d0
		move.l	active_buffer(a6),d1
		move.l	d1,active_screen(a6)
		move.l	d0,active_buffer(a6)
		rts

*The most important routine. As you can see I skip the first two lines, because there are no colours
*under them to get the complete average value from. 
*Note that there is no real bounds checking. Not neccesary.
calc_buffer:
		lea.l	variables(pc),a6
		move.l	active_buffer(a6),a3
		move.l	active_screen(a6),a2
		add.l	#dim_x-1,a2				*jump lines (or we get wrong average)
		add.l	#2*dim_x,a3				*start one line higher in other buffer to make fire go up, up and away
		moveq	#0,d4					*supposedly faster than clr
		move.l	#(dim_y-2)*dim_x,d7
calc_loop:
		moveq	#0,d5					*remove this at your peril
		moveq	#0,d4
		move.b	(a2),d5					*byte 1
		move.b	1(a2),d4				*byte 2
		add.w	d4,d5
		move.b	dim_x+1(a2),d4			*byte 3
		add.w	d4,d5
		divs	#3,d5
		beq		insert					*DON'T FADE IF ZERO!
		subi.b	#1,d5
insert:
		move.b	d5,(a3)+				*put in buffer
		add.w	#1,a2					*go to next byte
		dbra	d7,calc_loop			*do whole buffer
		rts

*Fills the two lowest lines with either $ff or $00 (red or black)
*If you make FREQUENCY lower the fire will be higher and vice versa
*It might seem silly, but I thought it was a waste of time to BSR to Random in the library
*as it just calls $a861, so I put it straight in.
*And why does the random routine alway get the same numbers (check it with Macsbug)?
set_random:
		lea.l	variables(pc),a6
		move.l	active_buffer(a6),a3
		move.w	#2*dim_x,d7				*do two lowest lines
rand_loop:
		clr.w	-(sp)
		dc.w	Random					*get random number
		move.w	(sp)+,d0
		andi.w	#$f,d0					*scale to x<16
		cmp.w	#frequency,d0			*fifty/fifty chance of red or black
		bls		less
		move.w	#red,d0					*greater than frequency? -> put a red one
		bra		do_loop
less:
		move.w	#black,d0
do_loop:
		move.b	d0,(a3)+
		dbra	d7,rand_loop
		rts		


*Plot buffer to screen
*I don't plot the first 5 lines as they will always be ugly ... saves a bit of time as well
*I don't use loops for the plotting because it saves time
*Note the modified plot routine.
draw_screen:
		lea.l	variables(pc),a6
		move.l	active_screen(a6),a3
		add.l	#5*dim_x,a3				*jump ugly lines
		move.w	#start_x,screen_x(a6)	*start at x
		move.w	#start_y,screen_y(a6)	*start at y
		move.w	#(dim_y-5)*dim_x,d7		*pixels to plot
draw_loop:
		moveq	#0,d0
		move.b	(a3)+,d0				*get colour number
		muls.l	#$1010101,d0			*extend to four pixels(is there a faster way? shifts are slower)
		move.w	screen_x(a6),d1			*get x + y
		move.w	screen_y(a6),d2
*Plot_256 from graphics library, modified to 4*4 pixels
		move.l	y_tab(pc),a0			*address of the start of each screen line
		move.l	0(a0,d2.w*4),a1			*vram address of the screen line
		add.w	d1,a1					*add x
		move.l	d0,(a1)					*plot it
		add.l	#s_width,a1				*next line
		move.l	d0,(a1)
		add.l	#s_width,a1
		move.l	d0,(a1)
		add.l	#s_width,a1
		move.l	d0,(a1)
*the following could probably be simplified since we're plotting full-screen
noplot:
		cmp.w	#start_x+4*dim_x,screen_x(a6)	*border check
		bne		cont_line
		move.w	#start_x-4,screen_x(a6)
		subi.w	#4,screen_y(a6)
cont_line:
		addi.w	#4,screen_x(a6)			*increment for next pixel
		dbra	d7,draw_loop
		rts

variables:
screen_x:		equ		0				*x and y for pixel plotting (word)
screen_y:		equ		2				*(word)
CLUT_pos:		equ		4				*clut is at this address (long)
active_buffer:	equ		8				*pointer to buffer (long)
active_screen:	equ		12				*(long)

		global	fire_demo
		extern	init_mac,a5_init,init_graph_lib,getkey_turbo,CLS_256
		extern	hide_mouse,show_mouse,getmem,lockmem,releasemem
		extern	y_tab
